#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <string>
#ifdef HAVE_GL_GL_H
#  include <GL/gl.h>
#elif HAVE_OPENGL_GL_H
#  include <OpenGL/gl.h>
#endif
#ifdef HAVE_GL_GLU_H
#  include <GL/glu.h>
#elif HAVE_OPENGL_GLU_H
#  include <OpenGL/glu.h>
#endif
#include <netcdf.h>
#include "height_quantity.hh"

using std::string;

HeightQuantity::HeightQuantity(const string &name,
			       GLdouble offset, GLdouble scale,
			       GLfloat red, GLfloat green, GLfloat blue):
  offset(offset), scale(scale), red(red), green(green), blue(blue), name(name),
  frame_number(-1){
  GLenum glerror;
  this->display_list = glGenLists(1);
  if((glerror = glGetError()) != GL_NO_ERROR) throw gluErrorString(glerror);
}

HeightQuantity::~HeightQuantity(void){
  if(glIsList(this->display_list) == GL_TRUE)
    glDeleteLists(this->display_list, 1);
}

void HeightQuantity::draw(int frame){
  if(!this->dynamic) frame = 0;
  if(this->frame_number != frame) this->compile(frame);
  this->frame_number = frame;
  glPushMatrix();
  glScalef(1, 1, this->scale);
  glTranslatef(0, 0, this->offset);
  glColor3f(this->red, this->green, this->blue);
  glCallList(this->display_list);
  glPopMatrix();
}

void HeightQuantity::set_SWWFile(const shared_ptr<SWWFile> &file){
  int ncstatus;
  this->sww_file = file;
  this->points = \
    shared_array<float>(new float[this->sww_file->number_of_points]);
  this->dynamic = this->sww_file->nc_inq_varndims_by_name(this->name) == 2;
  if((ncstatus = nc_inq_varid(this->sww_file->netcdf_id, this->name.c_str(),
			      &this->varid)) != NC_NOERR)
    throw nc_strerror(ncstatus);
}

void HeightQuantity::compile(int frame){
  size_t start_1[1] = {0};
  size_t start_2[2] = {0, 0};
  size_t span_1[1];
  size_t span_2[2] = {1, 0};
  int ncstatus;
  if(this->dynamic){
    span_2[1] = this->sww_file->number_of_points;
    ncstatus = nc_get_vara_float(this->sww_file->netcdf_id, this->varid,
				 start_2, span_2, this->points.get());
  }else{
    span_1[0] = this->sww_file->number_of_points;
    ncstatus = nc_get_vara_float(this->sww_file->netcdf_id, this->varid,
				 start_1, span_1, this->points.get());
  }
  if(ncstatus != NC_NOERR) throw nc_strerror(ncstatus);
  /* TODO: Compile the triangles into an OGL displaylist. */
}
